@@ -6,11 +6,11 @@ import os
 import sys
 import logging
 import configparser
-
-from tornado.stack_context import ExceptionStackContext
+import asyncio
 
 from .lib import nicelogger
 from .get_version import get_version
+from .source import session
 
 from . import __version__
 
@@ -41,6 +41,9 @@ def safe_overwrite(fname, data, *, method='write', mode='w', encoding=None):
   # if not using "with", write can fail without exception
   with open(tmpname, mode, encoding=encoding) as f:
     getattr(f, method)(data)
+    # see also: https://thunk.org/tytso/blog/2009/03/15/dont-fear-the-fsync/
+    f.flush()
+    os.fsync(f.fileno())
   # if the above write failed (because disk is full etc), the old data should be kept
   os.rename(tmpname, fname)
 
@@ -60,9 +63,8 @@ def write_verfile(file, versions):
   safe_overwrite(file, data, method='writelines')
 
 class Source:
-  started = False
-  tasks = 0
   oldver = newver = None
+
   def __init__(self, file):
     self.config = config = configparser.ConfigParser(
       dict_type=dict, allow_no_value=True
@@ -71,63 +73,72 @@ class Source:
     config.read_file(file)
     if '__config__' in config:
       c = config['__config__']
-      d = os.path.dirname(file.name)
-      self.oldver = os.path.expandvars(os.path.expanduser(os.path.join(d, c.get('oldver'))))
-      self.newver = os.path.expandvars(os.path.expanduser(os.path.join(d, c.get('newver'))))
 
-  def check(self):
-    self.started = True
+      if 'oldver' in c and 'newver' in c:
+        d = os.path.dirname(file.name)
+        self.oldver = os.path.expandvars(os.path.expanduser(
+          os.path.join(d, c.get('oldver'))))
+        self.newver = os.path.expandvars(os.path.expanduser(
+          os.path.join(d, c.get('newver'))))
 
+      self.max_concurrent = c.getint('max_concurrent', 20)
+      session.nv_config = config["__config__"]
+
+    else:
+      self.max_concurrent = 20
+
+  async def check(self):
     if self.oldver:
       self.oldvers = read_verfile(self.oldver)
     else:
       self.oldvers = {}
     self.curvers = self.oldvers.copy()
 
+    futures = set()
     config = self.config
     for name in config.sections():
       if name == '__config__':
         continue
-      self.task_inc()
       conf = config[name]
       conf['oldver'] = self.oldvers.get(name, None)
-      with ExceptionStackContext(self._handle_exception):
-        get_version(name, conf, self.print_version_update)
+      fu = asyncio.ensure_future(get_version(name, conf))
+      fu.name = name
+      futures.add(fu)
 
-  def _handle_exception(self, type, value, traceback):
-    self.task_dec()
-    raise value.with_traceback(traceback)
+      if len(futures) >= self.max_concurrent:
+        (done, futures) = await asyncio.wait(
+          futures, return_when = asyncio.FIRST_COMPLETED)
+        for fu in done:
+          self.future_done(fu)
 
-  def task_inc(self):
-    self.tasks += 1
+    if futures:
+      (done, _) = await asyncio.wait(futures)
+      for fu in done:
+        self.future_done(fu)
 
-  def task_dec(self):
-    self.tasks -= 1
-    if self.tasks == 0 and self.started:
-      if self.newver:
-        write_verfile(self.newver, self.curvers)
-      self.on_finish()
+    if self.newver:
+      write_verfile(self.newver, self.curvers)
+
+  def future_done(self, fu):
+    name = fu.name
+    try:
+      version = fu.result()
+      if version is not None:
+        self.print_version_update(name, version)
+    except Exception:
+      logger.exception('unexpected error happened with %s', name)
 
   def print_version_update(self, name, version):
-    try:
-      if version is None:
-        return
-
-      oldver = self.oldvers.get(name, None)
-      if not oldver or oldver != version:
-        logger.info('%s updated version %s', name, version)
-        self.curvers[name] = version
-        self.on_update(name, version, oldver)
-      else:
-        logger.debug('%s current version %s', name, version)
-    finally:
-      self.task_dec()
+    oldver = self.oldvers.get(name, None)
+    if not oldver or oldver != version:
+      logger.info('%s updated to %s', name, version)
+      self.curvers[name] = version
+      self.on_update(name, version, oldver)
+    else:
+      logger.debug('%s current %s', name, version)
 
   def on_update(self, name, version, oldver):
     pass
 
-  def on_finish(self):
-    pass
-
   def __repr__(self):
     return '<Source from %r>' % self.name
