@@ -1,32 +1,21 @@
 # MIT licensed
 # Copyright (c) 2013-2017 lilydjwg <lilydjwg@gmail.com>, et al.
 
-from functools import partial
 import logging
-import json
-
-from tornado.httpclient import AsyncHTTPClient
+from . import session
 
 logger = logging.getLogger(__name__)
 
-URL = 'https://www.archlinux.org/packages/search/json/?name='
+URL = 'https://www.archlinux.org/packages/search/json/'
 
-def get_version(name, conf, callback):
+async def get_version(name, conf):
   pkg = conf.get('archpkg') or name
   strip_release = conf.getboolean('strip-release', False)
-  url = URL + pkg
-  AsyncHTTPClient().fetch(
-    url, partial(_pkg_done, name, strip_release, callback))
-
-def _pkg_done(name, strip_release, callback, res):
-  if res.error:
-    raise res.error
-
-  data = json.loads(res.body.decode('utf-8'))
+  async with session.get(URL, params={"name": pkg}) as res:
+    data = await res.json()
 
   if not data['results']:
     logger.error('Arch package not found: %s', name)
-    callback(name, None)
     return
 
   r = [r for r in data['results'] if r['repo'] != 'testing'][0]
@@ -35,4 +24,4 @@ def _pkg_done(name, strip_release, callback, res):
   else:
     version = r['pkgver'] + '-' + r['pkgrel']
 
-  callback(name, version)
+  return version
